#include <stdio.h>
#include <stdlib.h>
#include <stralloc.h>
#include <string.h>
#include "person.h"
#include "hash.h"
#include "readline.h"

struct Person* search_person(PersonHashTable* pht, const char* name,
      unsigned short begin_year, unsigned short end_year) {
   const char* lastname = strrchr(name, ' ');
   if (lastname) {
      ++lastname;
   } else {
      lastname = name;
   }
   pht_first(pht, lastname);
   unsigned int given_name_len = lastname - name;
   struct Person* found = 0;
   struct Person* person = 0;
   bool not_in_range = false;
   while ((person = pht_next(pht))) {
      if (name != lastname &&
	    strncmp(name, person->name.s, given_name_len) != 0) {
	 continue;
      }
      if (begin_year && end_year &&
	    (person->died < begin_year || person->born > end_year)) {
	 not_in_range = true;
	 continue;
      }
      if (found) {
	 printf("%s is ambiguous.\n", name);
	 return 0;
      }
      found = person;
   }
   if (!found) {
      if (not_in_range) {
	 printf("Nobody found with that name living in the time "
	    "from %d to %d.\n", begin_year, end_year);
      } else {
	 printf("Nobody found with that name.\n");
      }
   }
   return found;
}

int main() {
   /* load composers from composers.txt */
   PersonHashTable* pht = pht_create();
   if (!pht) exit(1);
   FILE* fp = fopen("composers.txt", "r");
   if (!fp) exit(2);

   unsigned int count = 0;
   unsigned int min_year = 0;
   unsigned int max_year = 0;
   struct Person* p;
   while ((p = read_person(fp))) {
      if (!min_year || p->born < min_year) min_year = p->born;
      if (!max_year || p->died > max_year) max_year = p->died;
      pht_add(pht, p); ++count;
   }
   printf("%d dead composers loaded", count);
   if (min_year && max_year) {
      printf(" which lived in the time from %d to %d", min_year, max_year);
   }
   printf(".\n");

   printf(
      "*** Composer Chain Game ***\n"
      "Start by naming a composer and then name another one\n"
      "whose live span intersects that of the former composer.\n"
      "You get a point for each extension of the chain. No composer\n"
      "may be named twice.\n"
   );

   unsigned int chain_count = 0;
   stralloc line = {0};
   struct Person* last_person = 0;
   while (printf("Name: "), readline(stdin, &line)) {
      stralloc_0(&line);
      struct Person* p;
      if (last_person) {
	 p = search_person(pht, line.s, last_person->born, last_person->died);
      } else {
         p = search_person(pht, line.s, 0, 0);
      }
      if (p) {
	 if (p->named) {
	    printf("Already named: ");
	 } else {
	    p->named = true;
	    if (last_person) ++chain_count;
	    last_person = p;
	 }
	 write_person(stdout, p);
      }
   }
   if (chain_count) {
      printf("You got %d points.\n", chain_count);
   }

   pht_free(pht);
}
