#include <stdlib.h>
#include <string.h>
#include "hash.h"

#define BUCKET_SIZE 8192

struct PersonHashTableEntry {
   struct Person* person;
   struct PersonHashTableEntry* next;
};

struct PersonHashTable {
   struct PersonHashTableEntry* bucket[BUCKET_SIZE];
   struct PersonHashTableEntry* it;
   const char* it_name; // name given to pht_first
};

/* hash algorithm stolen from Dan Bernstein's cdbhash.c */
#define HASHSTART 5381

static unsigned int hashadd(int hashval, unsigned char ch) {
   hashval += hashval << 5;
   return hashval ^ ch;
}

static unsigned int compute_hash(const char* buf) {
   int hashval = HASHSTART;
   while (*buf) {
      hashval = hashadd(hashval, *buf++);
   }
   return hashval;
}

PersonHashTable* pht_create() {
   PersonHashTable* pht = calloc(1, sizeof(PersonHashTable));
   return pht;
}

bool pht_add(PersonHashTable* pht, struct Person* person) {
   struct PersonHashTableEntry* entry =
      malloc(sizeof(struct PersonHashTableEntry));
   if (!entry) return false;
   entry->person = person;
   unsigned int index = compute_hash(person->lastname) % BUCKET_SIZE;
   entry->next = pht->bucket[index];
   pht->bucket[index] = entry;
   return true;
}

void pht_first(PersonHashTable* pht, const char* lastname) {
   unsigned int index = compute_hash(lastname) % BUCKET_SIZE;
   pht->it_name = lastname;
   pht->it = pht->bucket[index];
}

struct Person* pht_next(PersonHashTable* pht) {
   while (pht->it) {
      struct Person* p = pht->it->person;
      pht->it = pht->it->next;
      if (strcmp(p->lastname, pht->it_name) == 0) {
	 return p;
      }
   }
   return 0;
}

void pht_free(PersonHashTable* pht) {
   for (int i = 0; i < BUCKET_SIZE; ++i) {
      struct PersonHashTableEntry* entry = pht->bucket[i];
      if (entry) {
	 pht->bucket[i] = 0;
	 struct PersonHashTableEntry* next;
	 while (entry) {
	    next = entry->next;
	    free(entry->person);
	    free(entry);
	    entry = next;
	 }
      }
   }
}
