#include <ctype.h>
#include <stdlib.h>
#include "person.h"

/* read a four-digit-year, return 0 in case of errors or EOF */
static unsigned short read_year(FILE* fp) {
   int ch;
   unsigned int val = 0;
   while ((ch = getc(fp)) != EOF && isdigit(ch)) {
      val = val * 10 + ch - '0';
      if (val >= 10000) return 0; /* does not look like a valid year */
   }
   if (ch == EOF) return 0;
   ungetc(ch, fp);
   return val;
}

/* reads an input line from fp, consisting of
   the full name followed by a space, the year
   of birth, a space, and the year when the
   person died */
struct Person* read_person(FILE* fp) {
   stralloc name = {0};
   int ch;
   while ((ch = getc(fp)) != EOF && ch != '\n' && !isdigit(ch)) {
      if (!stralloc_readyplus(&name, 1)) {
	 stralloc_free(&name); return 0;
      }
      name.s[name.len++] = ch;
   }
   /* remove trailing whitespace */
   while (name.len > 0 && isspace(name.s[name.len - 1])) {
      --name.len;
   }
   /* check input for validity and append terminating null-byte */
   if (ch == EOF || name.len == 0 || !isdigit(ch) || !stralloc_0(&name)) {
      stralloc_free(&name); return 0;
   }
   ungetc(ch, fp);
   /* look for begin of lastname */
   char* lastname = 0;
   for (int index = name.len - 1; index >= 0; --index) {
      if (isspace(name.s[index])) {
	 lastname = name.s + index + 1; break;
      }
   }
   if (!lastname) lastname = name.s;
   unsigned short born, died;
   struct Person* person;
   if ((born = read_year(fp)) && (getc(fp) == '-') &&
	 (died = read_year(fp)) && (getc(fp) == '\n') &&
	 (person = malloc(sizeof(struct Person)))) {
      person->name = name;
      person->lastname = lastname;
      person->born = born;
      person->died = died;
      person->named = false;
      return person;
   } else {
      stralloc_free(&name); return 0;
   }
}

/* write a person to fp in the input format, followed by a newline */
bool write_person(FILE* fp, struct Person* person) {
   return fprintf(fp,
      "%s %d-%d\n", person->name.s, person->born, person->died) > 0;
}

/* release storage associated with person */
void free_person(struct Person* person) {
   stralloc_free(&person->name);
   free(person);
}
